--[[
--------------------------------------------------------------------------------
    calca-Plus v2.0
    
    Requires transmitter firmware 4.22 or higher.
    
    Lua application for JETI DC/DS transmitters

	Copyright (c) 2024, Walter Loetscher
	All rights reserved.

	License: Share alike
	Can be used and changed non commercial
--]]
collectgarbage()
--------------------------------------------------------------------------------
-- Locals for application
local dix11, warning, delay, sAltEn
local voltSe, voltSeId, voltSePa
local ampSe, ampSeId, ampSePa
local saf, rst, ann, acs, sNumber
local volt = {index = 0, added = 0, ref = 0, calc = 0, comp = 0, smoo = 0, raw = 0, secraw = 0}
local thr = {t00 = 0, t25 = 0, t50 = 0, t75 = 0, t100 = 0, cor25 = 0, cor50 = 0, cor75 = 0, cor100 = 0}
local throttle = {pointer, value = 0.0, percent = 0, avg = 0, added = 0}
local count = {loop = 0, s2 = 0, c1 = 30, c2 = 30, c3 = 30, c4 = 30, c5 = 30}
local calcPerc = {raw = 100.00, int = 100, fac = 1.00, step = 5}
local reduction = {zero=0,one=0,two=0,three=0,four=0,five=0,perc=0,total=0,power=0,long=0,medium=0,short=0}
local amp = {raw=0, zero=0.01, koe = 0.93, one=0,two=0,three=0,four=0,five=0, sum1 = 0, sum2 = 0, sum3 = 0, sum4 = 0, sum5 = 0}
local timer = {actual = 0, second = 0, learnDelay = 0, flagMax = 100, flag1 = 0, flag2 = 0}
local energy = {long = 0, medium = 0, short = 0}
local koe= {short = 0.95, medium = 0.3, long = 0.15, power = 0.45, learn = 0.2}			-- learn von 0.3 auf 0.2, medium von 0.45 auf 0.4 powwer von 0.8 auf 0.6 short von 1.15 auf 1
local c_Rate = {dyn = 0, engine = 0, batt = 0, battA = 0, battB = 0, corrA = 0, corrB = 0, storeA = 0, storeB = 0, changeA = false, changeB = false}
local delta = {volt = 0, avg = 0, prog = 0, count = 0, sum = 0}
local capacity = {battA = 0, battB = 0, used = 0, loaded = 0, actual = 0, set = false, valid = true}
local switch, p, c, v, f, k = {}, {}, {}, {}, {}, {}
local alarm = {calca = 0, volt = 0, temp = 0, pre = 0, st50 = true, stPre = false, stCalc = true, stVolt = false, stTemp = true}
local learn = {step = 1, message = "No Volt!", rVolt = 0, sumVolt = 0, one = 0, two = 0, three = 0, four = 0, five = 0, dCalc = 0}

local versNumber, oldversNumber, state, values, change = 0, 0, 0, true, true
local batTemp, adjust, toggle, toggCount = 20.0,0, true, 50
local color, tmp = 0,0

local sensorLalist = { "..." }
local sensorIdlist = { "..." }
local sensorPalist = { "..." }
--------------------------------------------------------------------------------
-- Read and set translations
local function setLanguage()
    local lng=system.getLocale()
    local file = io.readall("Apps/calca-Plus/calca-Plus.jsn")
    local obj = json.decode(file)
    if(obj) then
        dix11 = obj[lng] or obj[obj.default]
    end
end
--------------------------------------------------------------------------------
-- Telemetry and Data Verification
local function printTelem()
	if throttle.value and (throttle.value ~= 0.0) then
		if versNumber == 2 then
	-- Learning	
			lcd.setColor(255,255,255)
			lcd.drawFilledRectangle(0,0,150,68,255)
			if count.c1 == 0 then lcd.setColor(0,255,0,80) else lcd.setColor(255,0,0,80) end		
			lcd.drawFilledRectangle(5,5,25,25)
			if count.c2 == 0 then lcd.setColor(0,255,0,80) else lcd.setColor(255,0,0,80) end
			lcd.drawFilledRectangle(34,5,25,25)
			if count.c3 == 0 then lcd.setColor(0,255,0,80) else lcd.setColor(255,0,0,80) end
			lcd.drawFilledRectangle(63,5,25,25)
			if count.c4 == 0 then lcd.setColor(0,255,0,80) else lcd.setColor(255,0,0,80) end
			lcd.drawFilledRectangle(92,5,25,25)
			if count.c5 == 0 then lcd.setColor(0,255,0,80) else lcd.setColor(255,0,0,80) end
			lcd.drawFilledRectangle(121,5,25,25)
			lcd.setColor(0,0,0)
			lcd.drawNumber(10,7,count.c1)
			lcd.drawNumber(39,7,count.c2)
			lcd.drawNumber(68,7,count.c3)
			lcd.drawNumber(97,7,count.c4)
			lcd.drawNumber(126,7,count.c5)			
			lcd.drawText(5,35, learn.message, FONT_NORMAL)
			lcd.drawText(110,50,string.format("%.3s%%",throttle.percent),FONT_NORMAL)			
			lcd.drawText(94,55, "V", FONT_MINI)
			lcd.drawText(100,55, "A", FONT_MINI)			
			if volt.raw > 3.2 then lcd.setColor(0,255,0,80) else lcd.setColor(255,0,0,80) end
			lcd.drawFilledRectangle(94,55,6,10)
			if amp.raw > 0 then lcd.setColor(0,255,0,80) else lcd.setColor(255,0,0,80) end
			lcd.drawFilledRectangle(100,55,6,10)			
			lcd.setColor(0,0,0)
			lcd.drawText(0,55,string.format("Red: %.2f",reduction.zero),FONT_MINI)
			lcd.drawText(50,55,string.format("dC: %.2f",learn.dCalc),FONT_MINI)
	-- Normal
		else
		-- Color
			lcd.setColor(255,255,255)														-- Background
			lcd.drawFilledRectangle(0,0,150,68,255)
			if calcPerc.raw <= alarm.calca then lcd.setColor(237,10,10)						-- Capacity			
			elseif calcPerc.raw <= alarm.pre then lcd.setColor(242,152,21)
			else lcd.setColor(50,212,27)
			end
			lcd.drawFilledRectangle(0,0,70,19,150)	
			if volt.smoo <= (alarm.volt/100) then lcd.setColor(237,10,10)					-- Cell Voltage	Color		
			elseif volt.smoo <= ((alarm.volt/100)+0.05) then lcd.setColor(242,152,21)
			else lcd.setColor(50,212,27)
			end
			if versNumber ~= 3 then															-- Cell Voltage
			lcd.drawFilledRectangle(0,20,106,30,150) else
			lcd.drawFilledRectangle(0,20,106,30,25)
			end
			if capacity.valid == false then
			lcd.setColor(0,0,255)
			lcd.drawFilledRectangle(71,0,84,19,130)
			else
			lcd.setColor(20,200,20)
			lcd.drawFilledRectangle(71,0,84,19,150)			
			end
			if batTemp >= alarm.temp then lcd.setColor(237,10,10)							-- Bat Temp		
			elseif batTemp >= alarm.temp - 10 then lcd.setColor(242,152,21)
			else lcd.setColor(50,212,27)
			end
			lcd.drawFilledRectangle(120,51,30,17,150)							
			lcd.drawFilledRectangle(0,51,120,17,150)
			if saf == 1 then																-- Safety Switch
			lcd.setColor(0,0,255)
			lcd.drawFilledRectangle(107,20,45,30,130)
			else
			lcd.setColor(20,200,20)
			lcd.drawFilledRectangle(107,20,45,30,150)
			lcd.setColor(0,0,0)
			end
		-- Text			
			lcd.setColor(0,0,0)																-- Color Text			
			lcd.drawText(0,5,dix11.cap,FONT_MINI)											-- Capacity	
			lcd.drawText(25,0,string.format("%.4s%%",calcPerc.raw),FONT_BOLD)			
			lcd.drawText(75,5,dix11.mAh,FONT_MINI)											-- Cap used
			lcd.drawText(83,0,string.format("%5dmAh",capacity.used),FONT_NORMAL)			
			if versNumber ~= 3 then															-- Cell Voltage
			lcd.drawText(0,38,dix11.cell,FONT_MINI)
			lcd.drawText(21,15,string.format("%.2fV",volt.smoo),FONT_MAXI) else
			lcd.drawText(18,25, dix11.noVolt, FONT_BOLD)
			end		
			lcd.drawText(107,22,dix11.throttle1,FONT_MINI)									-- Throttle
			lcd.drawText(110,32,string.format("%3d%%",throttle.percent),FONT_NORMAL)
			if sAltEn == 1 then lcd.drawText(0,55,"B",FONT_MINI) else lcd.drawText(0,55,"A",FONT_MINI) end
			lcd.drawText(8,50,string.format("%.20s",capacity.actual.."mAh "..c_Rate.batt.."C("..adjust..")"),FONT_NORMAL)	-- Bat
			lcd.drawText(132,55,string.format("%.0f°",math.floor(batTemp)),FONT_MINI)		-- Bat Temp
		end		
	else
		lcd.drawText(10,10, dix11.missing, FONT_NORMAL)
	end
	collectgarbage()
	if warning then print(warning) warning = "" end
end
--------------------------------------------------------------------------------
local function printDebug()																	--Second Telemetry Window
	lcd.drawText(80,0,string.format("rawV %.2f",volt.raw),FONT_MINI)
	lcd.drawText(80,9,string.format("delta%.2f",delta.volt),FONT_MINI)
--	lcd.drawText(5,0,string.format("secRaw %.2f",volt.secraw),FONT_MINI)
	lcd.drawText(5,0,string.format("longR %.2f",reduction.long),FONT_MINI)
	lcd.drawText(5,9,string.format("powerR %.2f",reduction.power),FONT_MINI)
	lcd.drawText(5,18,string.format("shortR %.2f",reduction.short),FONT_MINI)
	lcd.drawText(5,27,string.format("medR %.2f",reduction.medium),FONT_MINI)
	lcd.drawText(5,36,string.format("compV %.2f",volt.comp),FONT_MINI)
	lcd.drawText(5,45,string.format("smoo %.2f",volt.smoo),FONT_MINI)
	lcd.drawText(80,45,string.format("calcV %.2f",volt.calc),FONT_MINI)
	lcd.drawText(80,27,string.format("cRate %.2f",c_Rate.engine),FONT_MINI)
	lcd.drawText(80,18,string.format("dynCR %.2f",c_Rate.dyn),FONT_MINI)
	lcd.drawText(80,36,string.format("dAvg %.2f",delta.avg),FONT_MINI)
	lcd.drawText(5,54,string.format("Mode: %d",versNumber),FONT_MINI)
	lcd.drawText(80,54,string.format("State: %d",state),FONT_MINI)
end
--------------------------------------------------------------------------------
local function printInfo()																	--Second Telemetry Window
	lcd.drawText(0,5, dix11.minimas, FONT_MINI)
	lcd.drawText(0,20,string.format(dix11.cap2.." %d%%",alarm.calca),FONT_MINI)
	lcd.drawText(0,30,string.format(dix11.cellVolt.." %.2f",alarm.volt/100),FONT_MINI)
	lcd.drawText(0,40,string.format(dix11.tempAlarm2.." %d",alarm.temp),FONT_MINI)
	lcd.drawLine(75,3,75,65)
	lcd.drawText(80,5, dix11.infos, FONT_MINI)
	lcd.drawText(80,20,string.format("rawV: %.2f",volt.raw),FONT_MINI)
	lcd.drawText(80,30,string.format("calcV: %.2f",volt.calc),FONT_MINI)
	lcd.drawText(80,40,string.format("cRate: %.0f",c_Rate.engine),FONT_MINI)
	lcd.drawText(80,54,string.format("M:%d",versNumber),FONT_MINI)
	lcd.drawText(100,54,string.format("S:%d",state),FONT_MINI)
end
--------------------------------------------------------------------------------
-- Read available sensors for user to select
local function readSensors()
    local sensors = system.getSensors()
    local format = string.format
    local insert = table.insert
    for i, sensor in ipairs(sensors) do
        if (sensor.label ~= "") then
            insert(sensorLalist, format("%s", sensor.label))
            insert(sensorIdlist, format("%s", sensor.id))
            insert(sensorPalist, format("%s", sensor.param))
        end
    end
end
----------------------------------------------------------------------
local function setDef()																		-- Default Werte nach Erstinstallation
	thr.t00 = 10
	thr.t25 = capacity.battA * c_Rate.battA / 100 * 0.06
	thr.t50 = capacity.battA * c_Rate.battA / 100 * 0.21
	thr.t75 = capacity.battA * c_Rate.battA / 100 * 0.48
	thr.t100 = capacity.battA * c_Rate.battA / 100 * 0.7
	state = 9
end
----------------------------------------------------------------------
-- Actions when settings changed
local function vsChanged(value)																-- Volt Sensor
	if value > 0 then
		voltSe = value
		voltSeId = string.format("%s", sensorIdlist[voltSe])
		voltSePa = string.format("%s", sensorPalist[voltSe])
		if (voltSeId == "...") then
			voltSeId = 0
			voltSePa = 0 
		end
		system.pSave("voltSe", value)
		system.pSave("voltSeId", voltSeId)
		system.pSave("voltSePa", voltSePa)
	end	
end
local function asChanged(value)																-- Amp Sensor
	if value > 0 then
		ampSe = value
		ampSeId = string.format("%s", sensorIdlist[ampSe])
		ampSePa = string.format("%s", sensorPalist[ampSe])
		if (ampSeId == "...") then
			ampSeId = 0
			ampSePa = 0 
		end
		system.pSave("ampSe", value)
		system.pSave("ampSeId", ampSeId)
		system.pSave("ampSePa", ampSePa)
	end
end
local function cvChanged(value) 															-- 1=Auto 2=Learn 3=No Volt 4=Bat Switch
	oldversNumber = versNumber
	versNumber=value
	system.pSave("versNumber",value)
	values = true
	state = 9																				-- reset Learn
end
local function tcChanged(value)
	throttle.pointer=value
	system.pSave("throttle.pointer",value)
	if versNumber == 0 then setDef() cvChanged(1) form.reinit() end							-- Aenderung 30.4. state = 1 unnötig?
end
local function ssChanged(value) 															-- 1=Safe, -1=Disarmed, 0=not set
	switch.safe=value
	system.pSave("switch.safe",value)
end
local function rsChanged(value)
	switch.rep=value
	system.pSave("switch.rep",value)
end
local function nsChanged(value)
	switch.reset=value
	system.pSave("switch.reset",value)
end
local function acChanged(value)
	capacity.battA=value
	system.pSave("capacity.battA",value)
	values = true
end
local function bcChanged(value)
	capacity.battB=value
	system.pSave("capacity.battB",value)
	values = true
end
local function cnChanged(value)
	sNumber=value
	system.pSave("sNumber",value)
	values = true
end
local function craChanged(value)
	c_Rate.battA=value
	system.pSave("c_Rate.battA",value)
	values = true
	c_Rate.changeA = true
end
local function crbChanged(value)
	c_Rate.battB=value
	system.pSave("c_Rate.battB",value)
	values = true
	c_Rate.changeB = true
end
local function coraChanged(value)
	c_Rate.corrA=value
	system.pSave("c_Rate.corrA",value)
end
local function corbChanged(value)
	c_Rate.corrB=value
	system.pSave("c_Rate.corrB",value)
end
local function cwChanged(value)
	alarm.calca=value
	system.pSave("alarm.calca",value)
end
local function vaChanged(value)
	alarm.volt=value
	system.pSave("alarm.volt",value)
end
local function taChanged(value)
	alarm.temp=value
	system.pSave("alarm.temp",value)
end
local function T100Changed(value)
	thr.t100=value
	system.pSave("thr.t100",value)
	values = true
	capacity.set = false
end
local function T75Changed(value)
	thr.t75=value
	system.pSave("thr.t75",value)
	values = true
	capacity.set = false
end
local function T50Changed(value)
	thr.t50=value
	system.pSave("thr.t50",value)
	values = true
	capacity.set = false
end
local function T25Changed(value)
	thr.t25=value
	system.pSave("thr.t25",value)
	values = true
	capacity.set = false
end
local function T00Changed(value)
	thr.t00=value
	system.pSave("thr.t00",value)
	values = true
end
local function bsChanged(value)
	switch.batt=value
	system.pSave("switch.batt",value)
end
local function ucChanged(value)
	capacity.used=value
	system.pSave("capacity.used",value)
	capacity.set = true
end
local function lcChanged(value)
	capacity.loaded=value
	system.pSave("capacity.loaded",value)
	capacity.set = true
end
--------------------------------------------------------------------------------
local function calc_f()																		-- PowerReduction über die Laufzeit
	for i = 0, 50 do f[i] = 0 end
	f[0] = 0.65						-- 0.55
	f[10] = 0.7						-- 0.6
	f[20] = 0.75					-- 0.65
	f[30] = 0.8						-- 0.75
	f[40] = 0.85					-- 0.8	
	f[50] = 0.9						-- 0.85
	local fillfrom, fillto, gap = 1,1,0
	while fillfrom < 49 do
		while (f[fillfrom] > 0.0) and (fillfrom < 49) do
			fillfrom = fillfrom + 1			
		end
		fillto = fillfrom
		while f[fillto+1] == 0.0 do
			fillto = fillto + 1			
		end
			gap = (f[fillto+1] - f[fillfrom-1]) / (fillto - fillfrom + 2)
				for i = fillfrom, fillto do
				f[i] = f[i-1] + gap
				end
		fillfrom = fillto + 2
	end
end
--------------------------------------------------------------------------------
local function calc_v()																		-- Volt (3.2 to 4.2) vs Calca Percent V[0] is 3.2 Volt and 0%	Ausgangsbestimmung
	for i = 0, 50 do v[i] = 0 end
	v[0] = 0																				-- Volt Hundertstel zwischen 3,2 bis 4,2 Volt
	v[11] = 1
	v[16] = 5
	v[22] = 10																				-- entsprechen % CalCa
	v[26] = 20
	v[28] = 30
	v[29] = 40
	v[30] = 50
	v[32] = 60
	v[36] = 70
	v[39] = 80
	v[42] = 85
	v[44] = 90
	v[47] = 95
	v[50] = 100
	local fillfrom, fillto, gap = 1,1,0
	while fillfrom < 50 do		
		while (v[fillfrom] > 0.0) and (fillfrom < 49) do
			fillfrom = fillfrom + 1
		end
		fillto = fillfrom
		while v[fillto+1] == 0.0 do
			fillto = fillto + 1
		end		
			gap = (v[fillto+1] - v[fillfrom-1]) / (fillto - fillfrom + 2)
				for i = fillfrom, fillto do
				v[i] = v[i-1] + gap
				end
		fillfrom = fillto + 2
	end
end
--------------------------------------------------------------------------------
local function calc_c()																		-- Volt Hundertstel zwischen 3,2 bis 4,2 Volt
	for i = 0, 100 do c[i] = 0 end
	c[0] = 22 																				-- Volt Hundertstel zwischen 3,2 bis 4,2 Volt
	c[5] = 33
	c[10] = 44																				-- entsprechen % CalCa
	c[15] = 48
	c[20] = 52
	c[25] = 55
	c[30] = 57
	c[40] = 59
	c[50] = 61
	c[60] = 65
	c[70] = 73
	c[80] = 79
	c[85] = 84
	c[90] = 89
	c[95] = 94
	c[100] = 100	
	local fillfrom, fillto, gap = 1,1,0
	while fillfrom < 100 do		
		while (c[fillfrom] > 0.0) and (fillfrom < 99) do
			fillfrom = fillfrom + 1
		end
		fillto = fillfrom
		while c[fillto+1] == 0.0 do
			fillto = fillto + 1
		end		
			gap = (c[fillto+1] - c[fillfrom-1]) / (fillto - fillfrom + 2)
				for i = fillfrom, fillto do
				c[i] = c[i-1] + gap
				end
		fillfrom = fillto + 2
	end
end
--------------------------------------------------------------------------------
local function calc_p()																		-- PowerReduction über die Laufzeit
	for i = 0, 50 do p[i] = 0 end
	p[0] = thr.t00 * 10000 / capacity.actual / 3.6
	p[12] = thr.t25 * 100000 / capacity.actual / 3.6
	p[25] = thr.t50 * 100000 / capacity.actual / 3.6
	p[37] = thr.t75 * 100000 / capacity.actual / 3.6
	p[50] = thr.t100 * 100000 / capacity.actual / 3.6
	local fillfrom, fillto, gap = 1,1,0
	while fillfrom < 49 do
		while p[fillfrom] > 0.0 do
			fillfrom = fillfrom + 1
		end
		fillto = fillfrom
		while (p[fillto+1] == 0.0) and (fillto < 49) do
			fillto = fillto + 1
		end
			gap = (p[fillto+1] - p[fillfrom-1]) / (fillto - fillfrom + 2)
				for i = fillfrom, fillto do
				p[i] = p[i-1] + gap
				end
		fillfrom = fillto + 2
	end
	values = false
end
--------------------------------------------------------------------------------
local function calc_k()																		-- Coefficient for Learning Module (Reduction)
	for i = 0, 100 do k[i] = 0 end
	k[0] = 3
	k[25] = 1.5
	k[50] = 1.3
	k[75] = 1.2
	k[100] = 1.2	
	local fillfrom, fillto, gap = 1,1,0
	while fillfrom < 100 do
		while (k[fillfrom] > 0) and (fillfrom < 99) do
			fillfrom = fillfrom + 1
		end
		fillto = fillfrom
		while k[fillto+1] == 0.0 do
			fillto = fillto + 1
		end
			gap = (k[fillto+1] - k[fillfrom-1]) / (fillto - fillfrom + 2)
				for i = fillfrom, fillto do
				k[i] = k[i-1] + gap
				end
		fillfrom = fillto + 2
	end	
end
--------------------------------------------------------------------------------
local function ring()
	if (throttle.percent > 46) and (throttle.percent < 54) then system.playFile("/Apps/calca-Plus/ping80.wav", AUDIO_IMMEDIATE) end
	if (throttle.percent == 46) or (throttle.percent == 54) then system.playFile("/Apps/calca-Plus/ping78.wav", AUDIO_IMMEDIATE) end
	if (throttle.percent == 45) or (throttle.percent == 55) then system.playFile("/Apps/calca-Plus/ping76.wav", AUDIO_IMMEDIATE) end
	if (throttle.percent == 44) or (throttle.percent == 56) then system.playFile("/Apps/calca-Plus/ping74.wav", AUDIO_IMMEDIATE) end
	if (throttle.percent == 43) or (throttle.percent == 57) then system.playFile("/Apps/calca-Plus/silence.wav", AUDIO_IMMEDIATE) end
end
--------------------------------------------------------------------------------
local function calca()
	reduction.perc = (c[calcPerc.int]+320) / 420											-- Faktor für Spannungsreduktion über Laufzeit
	reduction.total = p[math.floor((throttle.avg/2)+0.5)]*reduction.perc					-- Reduktion für letzte Sekunde	
	calcPerc.raw = ((calcPerc.raw*10000)- reduction.total)/10000							-- Kapazität nach Reduktion
	if calcPerc.raw < 0 then calcPerc.raw = 0 end
	calcPerc.int = math.floor(calcPerc.raw + 0.5)
	calcPerc.fac = calcPerc.int / 100
end
--------------------------------------------------------------------------------
local function safeDatas()
	change = true
	system.pSave("sAltEn", sAltEn)
end
--------------------------------------------------------------------------------
local function checkSwitches()
	throttle.value = system.getInputsVal(throttle.pointer)
		if throttle.value then
		throttle.percent = math.floor((throttle.value * 100 + 100) / 2 + 0.5)
		end
	saf = (system.getInputsVal(switch.safe)) or 0
	rst = (system.getInputsVal(switch.reset)) or 0
	ann = (system.getInputsVal(switch.rep)) or 0
	acs = (system.getInputsVal(switch.batt)) or 0
	
	if acs == -1 and toggle == false then
		toggle = true
		sAltEn = 0
		safeDatas()
	end	
	if acs == 1 and toggle == true then		
		if sAltEn == 1 then
			sAltEn = 3
		elseif sAltEn == 0 then
			sAltEn = 2
		end		
		toggCount = toggCount - 1		
		if toggCount == 0 then 
			toggCount = 50
			toggle = false
			sAltEn = 1
			safeDatas()
		end
	end	
	if acs == -1 and sAltEn == 2 then
		toggCount = 50
		toggle = true
		sAltEn = 1
		safeDatas()
	end	
	if acs == -1 and sAltEn == 3 then
		toggCount = 50
		toggle = true
		sAltEn = 0
		safeDatas()
	end
	if sAltEn == 0 and acs == 1 then
		if toggle == false then 
		sAltEn = 1 
		safeDatas()
		end
	end		
	if sAltEn == 1 then capacity.actual = capacity.battB c_Rate.batt = c_Rate.battB			--switch to alt Battery
	else capacity.actual = capacity.battA c_Rate.batt = c_Rate.battA end
	if capacity.actual == 0 then capacity.actual = 25 end									-- avoid capacity zero
	if rst == 1 then calcPerc.raw=100 calcPerc.int=100 calcPerc.fac=1 end
	if ann == 1 then 
	delay = delay + 1
		if delay == 1 then 
		system.playNumber(calcPerc.raw, 0, "%") 
		system.playNumber(volt.smoo, 2, "V") 
		end
		if delay == 100 then delay = 0 end
	else
	delay = 0
	end
end
--------------------------------------------------------------------------------
local function getVoltAmp()
	local Voltage=system.getSensorByID(voltSeId,voltSePa)                      				-- if not valid replace by Zero
	if(Voltage and Voltage.valid) then 
		volt.raw=Voltage.value / sNumber
		timer.flag1 = 0
	else 
		if (volt.raw > 0) and (timer.flag1 < timer.flagMax) then timer.flag1 = timer.flag1 + 1
		else volt.raw = 0 timer.flag1 = 0
		end
	end
	local Current=system.getSensorByID(ampSeId,ampSePa)
	if(Current and Current.valid) then 
		amp.raw=Current.value
		timer.flag2 = 0
	else
		if (amp.raw > 0) and (timer.flag2 < timer.flagMax) then timer.flag2 = timer.flag2 + 1
		else amp.raw = 0 timer.flag2 = 0
		end
	end
end
--------------------------------------------------------------------------------
local function compV()
	energy.long = energy.long + reduction.total - 278										-- Abbau in 240 Minuten
	energy.medium = energy.medium + (reduction.total*2) - 300								-- Abbau in 60 Minuten
	energy.short = energy.short + (reduction.total * 3) - 1200								-- Abbau in 10 Minuten
	if energy.long < 1 then energy.long = 1 end
	if energy.medium < 1 then energy.medium = 1 end
	if energy.short < 1 then energy.short = 1 end
	if energy.medium > 250000 then energy.medium = 250000 end
	if energy.short > 60000 then energy.short = 60000 end
	c_Rate.dyn = math.floor((thr.t100 * reduction.total * 10000 / capacity.actual / p[50] / c_Rate.batt) + 0.5)
	reduction.power = koe.power * c_Rate.dyn * f[math.floor((calcPerc.int/50)+0.5)] / 100						-- von 0,8 auf 0,2
	reduction.short = koe.short * ((1 + calcPerc.fac) / 2) * energy.short / 1000000
	reduction.medium = koe.medium * energy.medium / 2500000
	reduction.long = koe.long * energy.long / 10000000
	volt.comp = volt.secraw + reduction.power + reduction.short + reduction.medium + reduction.long
end
--------------------------------------------------------------------------------
local function improve()
	if calcPerc.raw < calcPerc.step then
		calcPerc.step = calcPerc.step - 5
		delta.count = delta.count + 1
		delta.sum = delta.sum + delta.volt
		delta.avg = delta.sum / delta.count	
		if sAltEn == 1 then
		c_Rate.corrB = math.floor(((c_Rate.storeB + c_Rate.battB + (delta.avg * c_Rate.battB * 5))/2) + 0.5)
		corbChanged(c_Rate.corrB) adjust = c_Rate.corrB
		else 
		c_Rate.corrA = math.floor(((c_Rate.storeA + c_Rate.battA + (delta.avg * c_Rate.battA * 5))/2) + 0.5)
		coraChanged(c_Rate.corrA) adjust = c_Rate.corrA
		end
	end
end
--------------------------------------------------------------------------------
local function s2Delay()
	alarm.stPre = false
	count.s2 = count.s2 + 1
	if count.s2 == 100 then
		state  = 3 count.s2 = 0		
		volt.index = math.floor(((volt.comp - 3.20) * 50)+0.5)
		if volt.index > 50 then volt.index = 50 end
		if volt.index < 0 then volt.index = 0 end
		calcPerc.raw = v[volt.index]		
		calcPerc.int = math.floor(v[volt.index]+0.5)
		calcPerc.fac = calcPerc.int / 100
		calcPerc.step = calcPerc.raw - 7
		alarm.stPre = true
		if calcPerc.raw < 95 then capacity.valid = false end								-- usedCap entfernen
		if calcPerc.raw > alarm.calca + 2 then alarm.stCalc = true end
	end
end
--------------------------------------------------------------------------------
local function s4Delay()
	count.s2 = count.s2 + 1
	if count.s2 == 100 then state  = 3 count.s2 = 0 end
end
--------------------------------------------------------------------------------
local function warnings()
	if volt.smoo * 100  > alarm.volt + 3 then alarm.stVolt = true end						-- 3.Aug +3 from +2
	if calcPerc.raw <= 50 and alarm.st50 then
	system.playNumber(50, 0, "%")
	alarm.st50 = false
	print("50%")
	end	
	if alarm.stPre then
		if (calcPerc.raw <= (alarm.calca + 5)) or ((volt.smoo >= 2) and ((volt.smoo * 100) <= (alarm.volt + 3))) then		
		system.playFile("/Apps/calca-Plus/CalPreWarn.wav", AUDIO_QUEUE)						-- File Location noch bearbeiten!
		print("3-Klick: ", calcPerc.raw, volt.smoo)
		alarm.stPre = false
		end
	end
	if (calcPerc.raw <= alarm.calca) and alarm.stCalc then									-- KLammern???
	system.playFile("/Apps/calca-Plus/calWarnT.wav", AUDIO_IMMEDIATE)
	system.playNumber(calcPerc.raw, 0, "%")
	print("Alarm Capa: ", calcPerc.raw)
	alarm.stCalc = false
	end
	if (volt.smoo <= (alarm.volt + 1)/100) and ((state > 2) and alarm.stVolt) then			-- 3.Aug alarm.volt +1
	system.playFile("/Apps/calca-Plus/A_kuckuck_02.wav", AUDIO_IMMEDIATE)
	system.playNumber(volt.smoo, 2, "V")
	print("Alarm Volt: ", volt.smoo)
	alarm.stVolt = false
	end
	if (batTemp >= alarm.temp) and alarm.stTemp then
	system.playFile("/Apps/calca-Plus/CalBatTemp_en.wav", AUDIO_IMMEDIATE)
	print("Alarm Bat Temp: ", batTemp)
	alarm.stTemp = false
	end	
end
--------------------------------------------------------------------------------
local function rstLearn()
	count.c1 = 30 count.c2 = 30 count.c3 = 30 count.c4 = 30 count.c5 = 30
	reduction.zero=0 reduction.one=0 reduction.two=0 reduction.three=0 reduction.four=0 reduction.five=0
	amp.sum1 = 0 amp.sum2 = 0 amp.sum3 = 0 amp.sum4 = 0 amp.sum5 = 0
	learn.sumVolt = 0
	learn.step = 1
	T00Changed(thr.t00)
	T25Changed(thr.t25)
	T50Changed(thr.t50)
	T75Changed(thr.t75)
	T100Changed(thr.t100)
end
--------------------------------------------------------------------------------
local function signal(value)
	local stepL = value
	if (throttle.percent > (stepL - 2)) and (throttle.percent < (stepL +2)) then system.playFile("/Apps/calca-Plus/ping80.wav", AUDIO_IMMEDIATE) end
	if (throttle.percent == (stepL -2)) or (throttle.percent == (stepL + 2)) then system.playFile("/Apps/calca-Plus/ping78.wav", AUDIO_IMMEDIATE) end
	if (throttle.percent == (stepL - 3)) or (throttle.percent == (stepL + 3)) then system.playFile("/Apps/calca-Plus/ping76.wav", AUDIO_IMMEDIATE) end
	if (throttle.percent == (stepL - 4)) or (throttle.percent == (stepL + 4)) then system.playFile("/Apps/calca-Plus/silence.wav", AUDIO_IMMEDIATE) end
	if (throttle.percent == (stepL - 5)) or (throttle.percent == (stepL + 5)) then system.playFile("/Apps/calca-Plus/silence.wav", AUDIO_IMMEDIATE) end
end
--------------------------------------------------------------------------------
local function learning()
	local step6, step3 = 6, 3
	local ende = "/Apps/calca-Plus/CalPreWarn.wav"
	if learn.step == 1 then
		rstLearn()
		learn.message = dix11.learnIdle
		if throttle.percent == 0 then learn.message = dix11.learnSafe end
		if saf < 1 then learn.step = 2 end
	end					
	if learn.step == 2 then
		learn.message = dix11.learnIdle
		if throttle.percent < 6 then signal(0) end
		if throttle.percent < 2 then
			learn.sumVolt = learn.sumVolt + volt.raw
			amp.sum1 = amp.sum1 + amp.raw
			count.c1 = count.c1 - 1
			if count.c1 == 0 then
			system.playFile(ende, AUDIO_IMMEDIATE)
			learn.secVolt = learn.sumVolt / 30
			amp.one = amp.sum1 / 30
			learn.dCalc = volt.calc - learn.secVolt
				if amp.one > 0 then thr.t00 = amp.one * 100 else
				learn.one = 0.1
				thr.t00 = learn.one * 100
				end
			learn.step = 3 
			end
		end	
	end		
	if learn.step == 3 then
		reduction.zero = (volt.calc - learn.dCalc - volt.raw) * (1.2 - (calcPerc.raw * koe.learn / 100)) * k[throttle.percent]
--							Differenz							Koe für Akkuladung							Koe für Drosselstellung
		if reduction.zero < 0 then reduction.zero = 0 end
		if count.c2 > 0 then learn.message = dix11.learn25
		elseif count.c3 > 0 then learn.message = dix11.learn50
		elseif count.c4 > 0 then learn.message = dix11.learn75
		elseif count.c5 > 0 then learn.message = dix11.learnFull
		end	
		if (count.c2 > 0) and (throttle.percent > (25 - step6)) and (throttle.percent < (25 + step6)) then signal(25) end
		if (count.c2 > 0) and (throttle.percent > (25 - step3)) and (throttle.percent < (25 + step3)) then
			reduction.two = reduction.two + reduction.zero
			amp.sum2 = amp.sum2 + amp.raw
			count.c2 = count.c2 - 1
			if count.c2 == 0 then
			system.playFile(ende, AUDIO_IMMEDIATE)
			learn.two = reduction.two / 30 amp.two = amp.sum2 / 30
				if amp.two > 0 then thr.t25 = amp.two * 10 * amp.koe else
				thr.t25 = learn.two * capacity.actual / 100 * c_Rate.batt * amp.koe
				end
			end
		end
		if (count.c3 > 0) and (throttle.percent > (50 - step6)) and (throttle.percent < (50 + step6)) then signal(50) end
		if (count.c3 > 0) and (throttle.percent > (50 - step3)) and (throttle.percent < (50 + step3)) then
			reduction.three = reduction.three + reduction.zero
			amp.sum3 = amp.sum3 + amp.raw
			count.c3 = count.c3 - 1
			if count.c3 == 0 then
			system.playFile(ende, AUDIO_IMMEDIATE)
			learn.three = reduction.three / 30 amp.three = amp.sum3 / 30 
				if amp.three > 0 then thr.t50 = amp.three * 10 * amp.koe else
				thr.t50 = learn.three * capacity.actual / 100 * c_Rate.batt * amp.koe
				end
			end
		end		
		if (count.c4 > 0) and (throttle.percent > (75 - step6)) and (throttle.percent < (75 + step6)) then signal(75) end
		if (count.c4 > 0) and (throttle.percent > (75 - step3)) and (throttle.percent < (75 + step3)) then
			reduction.four = reduction.four + reduction.zero
			amp.sum4 = amp.sum4 + amp.raw
			count.c4 = count.c4 - 1
			if count.c4 == 0 then
			system.playFile(ende, AUDIO_IMMEDIATE)
			learn.four = reduction.four / 30 amp.four = amp.sum4 / 30 
				if amp.four > 0 then thr.t75 = amp.four * 10 * amp.koe else
				thr.t75 = learn.four * capacity.actual / 100 * c_Rate.batt * amp.koe
				end
			end
		end		
		if (count.c5 > 0) and (throttle.percent > (100 - step6)) and (throttle.percent < (100 + step6)) then signal(100) end
		if (count.c5 > 0) and (throttle.percent > (100 - step3)) and (throttle.percent < (100 + step3)) then
			reduction.five = reduction.five + reduction.zero
			amp.sum5 = amp.sum5 + amp.raw
			count.c5 = count.c5 - 1
			if count.c5 == 0 then
			system.playFile(ende, AUDIO_IMMEDIATE)
			learn.five = reduction.five / 30 amp.five = amp.sum5 / 30			
				if amp.five > 0 then thr.t100 = amp.five * 10 * amp.koe else
				thr.t100 = learn.five * capacity.actual / 100 * c_Rate.batt * amp.koe
				if thr.t100 < 1 then thr.t100 = 1 end
				end
			learn.step = 4
			end
		end				
	end
	if learn.step == 4 then
	if count.c2 > 0 then thr.t25 = thr.t100 * 0.09 end
	if thr.t25 < 3 then thr.t25 = thr.t100 * 0.09 end
	if count.c3 > 0 then thr.t50 = thr.t100 * 0.3 end
	if thr.t50 < 5 then thr.t50 = thr.t100 * 0.3 end
	if count.c4 > 0 then thr.t75 = thr.t100 * 0.68 end
	rstLearn()
	versNumber = oldversNumber
	system.pSave("versNumber",versNumber)
	learn.step = 1
	end	
end
--------------------------------------------------------------------------------
local function everySecond()
	timer.second = timer.actual	
	if saf < 1 then throttle.avg = math.floor(throttle.added/count.loop + 0.5) else throttle.avg = 0 end	--check Safety Switch	
	calca()
	if capacity.valid == true then
		if capacity.set == false then
		capacity.used = math.floor((100 - calcPerc.raw) * capacity.actual / 100)
		capacity.loaded = capacity.used
		else
		thr.cor100 = capacity.loaded / capacity.used * thr.t100 / 10
		thr.cor75 = capacity.loaded / capacity.used * thr.t75 / 10
		thr.cor50 = capacity.loaded / capacity.used * thr.t50 / 10
		thr.cor25 = capacity.loaded / capacity.used * thr.t25 / 10
		end
	else 
	capacity.used = 0000
	end	
	volt.calc = 3.2 + (c[calcPerc.int]/100)
	volt.secraw = volt.added / count.loop
	if volt.secraw > 0 then compV() else volt.comp = 0 end
	if state > 2 then
		if volt.comp > (volt.calc + delta.volt) then delta.volt = delta.volt + 0.001 end				
		if volt.comp < (volt.calc + delta.volt) then delta.volt = delta.volt - 0.001 end
		if volt.raw > 2 then improve() end
	end	
	if state > 1 then volt.smoo = volt.calc + delta.volt end	
	c_Rate.engine = math.floor(thr.t100 / capacity.actual * 100 + 0.5)	
	batTemp = batTemp + (c_Rate.dyn / 200) - 0.03
	if batTemp < 20 then batTemp = 20 end
	if volt.raw < 1 then learn.message = dix11.noVolt end									-- für Learn Modul
	throttle.added = 0
	volt.added = 0
	count.loop = 0
end
--------------------------------------------------------------------------------
local function loop()
	if c_Rate.changeA == true then coraChanged(0) c_Rate.changeA = false end
	if c_Rate.changeB == true then corbChanged(0) c_Rate.changeB = false end
	timer.actual = system.getTime()	
	if versNumber == 0 then return end
	if state == 9 then rstLearn() state = 1 end
	if (versNumber == 1 or versNumber == 4) and (state == 0) then 
		state = 1 checkSwitches() calc_p()													-- checkSwitches() rausgenommen
		alarm.stPre = true
		if sAltEn ~= 0 then tmp = sAltEn end
	end
	checkSwitches()
	if tmp ~= sAltEn then tmp = sAltEn calc_p() end	
	if versNumber ~= 3 then getVoltAmp() end	
	if values == true then calc_p() end
	if (state == 1) and (volt.raw > 3.2) then s2Delay() end
	if (state == 1) and (volt.raw <= 3.2) then count.s2 = 0 end
	if (state == 3) and (amp.raw > 0) then state = 4 end
	if (state > 2) and (volt.raw == 0) then state = 2 end
	if (state == 2) and (volt.raw > 3.2) then s4Delay() end 	
	warnings()	
	if (versNumber == 2) and (state > 2) then
	learning()
	end	
	count.loop = count.loop + 1
	volt.added = volt.added + volt.raw
	throttle.added = throttle.added + throttle.percent	
	if timer.actual >= timer.second + 1 then everySecond() end
    collectgarbage()
end
--------------------------------------------------------------------------------
-- Draw the main form (Application inteface)
local function initForm()
    local versTable = {dix11.auto,dix11.learn,dix11.novolt,dix11.batSwitch}
    local form, addRow, addLabel = form, form.addRow ,form.addLabel
    local addIntbox, addCheckbox = form.addIntbox, form.addCheckbox
    local addSelectbox, addInputbox = form.addSelectbox, form.addInputbox
	form.setTitle(dix11.appName.." "..dix11.version.."      W.L.")
	if versNumber == 0 then																	-- Aenderung 30.4.
	addLabel({label= "                        "..dix11.first, font=FONT_NORMAL})
	addRow(2)
	addLabel({label= dix11.capa1, font=FONT_BOLD})
	addIntbox(capacity.battA,25,20000,1500,0,25,acChanged, {font=FONT_BOLD})
	addRow(3)
	addLabel({label= dix11.cRate1, font=FONT_BOLD})
	addLabel({label= "("..string.format("%3d",c_Rate.corrA)..")", font=FONT_BOLD})
	addIntbox(c_Rate.battA,1,200,30,0,1,craChanged, {font=FONT_BOLD})
	addRow(2)
	addLabel({label= dix11.sNumber, font=FONT_BOLD})
	addIntbox(sNumber,1,14,3,0,1,cnChanged, {font=FONT_BOLD})
	addRow(2)
	addLabel({label= dix11.throttleControl, font=FONT_BOLD})
	addInputbox(throttle.pointer,true,tcChanged, {font=FONT_BOLD})
	addLabel({label=dix11.prop, font=FONT_MINI})
	else	
	addRow(2)
	addLabel({label= dix11.mode, font=FONT_BOLD})
	addSelectbox(versTable, versNumber, true, cvChanged, {font=FONT_BOLD})	
	addRow(2)
    addLabel({label=dix11.voltSensor, width=180, font=FONT_BOLD})
    addSelectbox(sensorLalist, voltSe, true, vsChanged, {font=FONT_BOLD})		
	addRow(2)
    addLabel({label=dix11.ampSensor, width=180, font=FONT_BOLD})							-- Versuch 180 statt 220
    addSelectbox(sensorLalist, ampSe, true, asChanged, {font=FONT_BOLD})	
	addRow(2)
	addLabel({label= dix11.throttleControl, font=FONT_BOLD})
	addInputbox(throttle.pointer,true,tcChanged, {font=FONT_BOLD})
	addLabel({label=dix11.prop, font=FONT_MINI})
	addRow(2)
	addLabel({label= dix11.safety, font=FONT_BOLD})
	addInputbox(switch.safe,true,ssChanged, {font=FONT_BOLD})
	addRow(2)
	addLabel({label= dix11.report, font=FONT_BOLD})
	addInputbox(switch.rep,true,rsChanged, {font=FONT_BOLD})
	addRow(2)
	addLabel({label= dix11.switchBat, font=FONT_BOLD})
	addInputbox(switch.batt,true,bsChanged, {font=FONT_BOLD})		
	addLabel({label= "----------------------------------------------------------"})
	addLabel({label= "                        "..dix11.warnings, font=FONT_NORMAL})	
	addRow(2)
	addLabel({label= dix11.calcAlarm, font=FONT_BOLD})
	addIntbox(alarm.calca,0,100,0,0,1,cwChanged, {font=FONT_BOLD})
	addRow(2)
	addLabel({label= dix11.voltAlarm, font=FONT_BOLD})
	addIntbox(alarm.volt,320,420,375,2,1,vaChanged, {font=FONT_BOLD})
	addRow(2)
	addLabel({label= dix11.tempAlarm, font=FONT_BOLD})
	addIntbox(alarm.temp,20,99,60,0,1,taChanged, {font=FONT_BOLD})	
	addLabel({label= "----------------------------------------------------------"})
	addLabel({label= "                        "..dix11.battery, font=FONT_NORMAL})	
	addRow(2)
	addLabel({label= dix11.sNumber, font=FONT_BOLD})
	addIntbox(sNumber,1,14,3,0,1,cnChanged, {font=FONT_BOLD})	
	addRow(2)
	addLabel({label= dix11.capa1, font=FONT_BOLD})
	addIntbox(capacity.battA,25,20000,1500,0,25,acChanged, {font=FONT_BOLD})
	addRow(2)
	addLabel({label= dix11.capa2, font=FONT_BOLD})
	addIntbox(capacity.battB,25,20000,0,0,25,bcChanged, {font=FONT_BOLD})					-- Minimum auf 25mAh gesetzt
	addRow(3)
	addLabel({label= dix11.cRate1, font=FONT_BOLD})
	addLabel({label= "("..string.format("%3d",c_Rate.corrA)..")", font=FONT_BOLD})
	addIntbox(c_Rate.battA,1,200,30,0,1,craChanged, {font=FONT_BOLD})
	addRow(3)
	addLabel({label= dix11.cRate2, font=FONT_BOLD})
	addLabel({label= "("..string.format("%3d",c_Rate.corrB)..")", font=FONT_BOLD})
	addIntbox(c_Rate.battB,1,200,30,0,1,crbChanged, {font=FONT_BOLD})
	addLabel({label= "----------------------------------------------------------"})
	addLabel({label= "                   "..dix11.consumption, font=FONT_NORMAL})	
	addRow(3)
	addLabel({label= dix11.throttle100, font=FONT_BOLD})
	addLabel({label= "("..string.format("%.1f",thr.cor100)..")", font=FONT_BOLD})
	addIntbox(thr.t100,1,10000,0,1,1,T100Changed, {font=FONT_BOLD})
	addRow(3)
	addLabel({label= dix11.throttle75, font=FONT_BOLD})
	addLabel({label= "("..string.format("%.1f",thr.cor75)..")", font=FONT_BOLD})	
	addIntbox(thr.t75,1,10000,0,1,1,T75Changed, {font=FONT_BOLD})
	addRow(3)
	addLabel({label= dix11.throttle50, font=FONT_BOLD})
	addLabel({label= "("..string.format("%.1f",thr.cor50)..")", font=FONT_BOLD})
	addIntbox(thr.t50,1,10000,0,1,1,T50Changed, {font=FONT_BOLD})
	addRow(3)
	addLabel({label= dix11.throttle25, font=FONT_BOLD})
	addLabel({label= "("..string.format("%.1f",thr.cor25)..")", font=FONT_BOLD})
	addIntbox(thr.t25,1,10000,0,1,1,T25Changed, {font=FONT_BOLD})
	addRow(2)
	addLabel({label= dix11.throttle0, font=FONT_BOLD})
	addIntbox(thr.t00,0,32767,0,2,1,T00Changed, {font=FONT_BOLD})	
	addLabel({label= "----------------------------------------------------------"})
	addLabel({label= "                        "..dix11.correction, font=FONT_NORMAL})	
	addRow(2)
	addLabel({label= dix11.used, font=FONT_BOLD})
	addIntbox(capacity.used,0,20000,0,0,1,ucChanged, {font=FONT_BOLD})
	addRow(2)
	addLabel({label= dix11.loaded, font=FONT_BOLD})
	addIntbox(capacity.loaded,0,20000,0,0,1,lcChanged, {font=FONT_BOLD})
	addRow(1)
    addLabel({label="     Powered by Swiss-Aerodesign.com   "..dix11.version.." ", font=FONT_MINI})
	end
end
--------------------------------------------------------------------------------
local function init()
	timer.actual = system.getTime()
	timer.second = timer.actual
    local pLoad = system.pLoad
	for i = 0, 50 do p[i] = 0 end
	calc_c()
	calc_v()
	calc_f()
	calc_k()	
    voltSe = pLoad("voltSe", 0)
    voltSeId = pLoad("voltSeId", 0)
    voltSePa = pLoad("voltSePa", 0)	
	ampSe = pLoad("ampSe", 0)
    ampSeId = pLoad("ampSeId", 0)
    ampSePa = pLoad("ampSePa", 0)
	throttle.pointer = pLoad("throttle.pointer",nil)								-- Throttle (P4 for Mode 2), use Proportional!
	switch.safe = pLoad("switch.safe",nil)											-- Safety Switch
	switch.rep = pLoad("switch.rep",nil)               	                        	-- Report actual Capacity
	switch.reset = pLoad("switch.reset",nil)               	                    	-- Reset Capacity	
	versNumber = pLoad("versNumber",0)												-- Version (Init, Auto, Learn, NoVolt, Bat Switch)
	capacity.battA = pLoad("capacity.battA",2000)									-- Capacity Akku 1
	capacity.battB = pLoad("capacity.battB",0)										-- Capacity Akku 2
	sNumber = pLoad("sNumber",3)												    -- Number of Cells
	c_Rate.battA = pLoad("c_Rate.battA",30)											-- C-Rate Batt 1
	c_Rate.battB = pLoad("c_Rate.battB",30)											-- C-Rate Batt 2
	c_Rate.corrA = pLoad("c_Rate.corrA",0)											-- gespeichert Korrektur C-Rate Batt A
	c_Rate.corrB = pLoad("c_Rate.corrB",0)											-- gespeichert Korrektur C-Rate Batt B
	alarm.calca = pLoad("alarm.calca",25)               							-- Final Warning
	alarm.volt = pLoad("alarm.volt",375)											-- Spannungs Alarm
	alarm.temp	= pLoad("alarm.temp",60)											-- Temperatur Alarm
	thr.t100 = pLoad("thr.t100",200)			                                	-- Current Throttle 100%
	thr.t75 = pLoad("thr.t75",100)			                                		-- Current Throttle 75%
	thr.t50 = pLoad("thr.t50",50)			                                		-- Current Throttle 50%
	thr.t25 = pLoad("thr.t25",20)			                                		-- Current Throttle 25%
	thr.t00 = pLoad("thr.t00",5)				                                	-- Current Throttle 0%
	switch.batt = pLoad("switch.batt")												-- Switch to other Battery
	sAltEn = pLoad("sAltEn",0)														-- 0 = primary Accu, 1 = alternate Accu
	capacity.used = pLoad("capacity.used",0)
	capacity.loaded = pLoad("capacity.loaded",0)
	if c_Rate.corrA > 0 then c_Rate.storeA = c_Rate.corrA else c_Rate.storeA = c_Rate.battA end	-- Zwischenspeicherung Korrektur C-Rate
	if c_Rate.corrB > 0 then c_Rate.storeB = c_Rate.corrB else c_Rate.storeB = c_Rate.battB end	
	system.registerForm(1,MENU_MAIN,dix11.appName,initForm)
	system.registerTelemetry(1,dix11.appName .." main ".. dix11.version,2,printTelem)
--	system.registerTelemetry(2,dix11.appName.." info ".. dix11.version,2,printDebug)
	system.registerTelemetry(2,dix11.appName.." info ".. dix11.version,2,printInfo)
    readSensors()
    collectgarbage()
end
--------------------------------------------------------------------------------
setLanguage()
collectgarbage()
return {init=init, loop=loop, author="Walter Loetscher", version=dix11.version, name=dix11.appName}
